@@ -3,7 +3,7 @@ use anyhow::Result;
 use crate::Client;
 
 pub struct Checks {
-    client: Client,
+    pub client: Client,
 }
 
 impl Checks {
@@ -43,10 +43,7 @@ impl Checks {
         );
 
         self.client
-            .post(
-                &url,
-                Some(reqwest::Body::from(serde_json::to_vec(body).unwrap())),
-            )
+            .post(&url, Some(reqwest::Body::from(serde_json::to_vec(body)?)))
             .await
     }
 
@@ -115,10 +112,7 @@ impl Checks {
         );
 
         self.client
-            .patch(
-                &url,
-                Some(reqwest::Body::from(serde_json::to_vec(body).unwrap())),
-            )
+            .patch(&url, Some(reqwest::Body::from(serde_json::to_vec(body)?)))
             .await
     }
 
@@ -147,26 +141,20 @@ impl Checks {
         per_page: i64,
         page: i64,
     ) -> Result<Vec<crate::types::CheckAnnotation>> {
-        let mut query = String::new();
-        let mut query_args: Vec<String> = Default::default();
+        let mut query_args: Vec<(String, String)> = Default::default();
         if page > 0 {
-            query_args.push(format!("page={}", page));
+            query_args.push(("page".to_string(), page.to_string()));
         }
         if per_page > 0 {
-            query_args.push(format!("per_page={}", per_page));
-        }
-        for (i, n) in query_args.iter().enumerate() {
-            if i > 0 {
-                query.push('&');
-            }
-            query.push_str(n);
+            query_args.push(("per_page".to_string(), per_page.to_string()));
         }
+        let query_ = serde_urlencoded::to_string(&query_args).unwrap();
         let url = format!(
             "/repos/{}/{}/check-runs/{}/annotations?{}",
             crate::progenitor_support::encode_path(&owner.to_string()),
             crate::progenitor_support::encode_path(&repo.to_string()),
             crate::progenitor_support::encode_path(&check_run_id.to_string()),
-            query
+            query_
         );
 
         self.client.get(&url, None).await
@@ -228,10 +216,7 @@ impl Checks {
         );
 
         self.client
-            .post(
-                &url,
-                Some(reqwest::Body::from(serde_json::to_vec(body).unwrap())),
-            )
+            .post(&url, Some(reqwest::Body::from(serde_json::to_vec(body)?)))
             .await
     }
 
@@ -262,10 +247,7 @@ impl Checks {
         );
 
         self.client
-            .patch(
-                &url,
-                Some(reqwest::Body::from(serde_json::to_vec(body).unwrap())),
-            )
+            .patch(&url, Some(reqwest::Body::from(serde_json::to_vec(body)?)))
             .await
     }
 
@@ -337,31 +319,29 @@ impl Checks {
         per_page: i64,
         page: i64,
     ) -> Result<crate::types::ChecksListRefResponse> {
-        let mut query = String::new();
-        let mut query_args: Vec<String> = Default::default();
+        let mut query_args: Vec<(String, String)> = Default::default();
         if !check_name.is_empty() {
-            query_args.push(format!("check_name={}", check_name));
+            query_args.push(("check_name".to_string(), check_name.to_string()));
+        }
+        if !filter.to_string().is_empty() {
+            query_args.push(("filter".to_string(), filter.to_string()));
         }
-        query_args.push(format!("filter={}", filter));
         if page > 0 {
-            query_args.push(format!("page={}", page));
+            query_args.push(("page".to_string(), page.to_string()));
         }
         if per_page > 0 {
-            query_args.push(format!("per_page={}", per_page));
+            query_args.push(("per_page".to_string(), per_page.to_string()));
         }
-        query_args.push(format!("status={}", status));
-        for (i, n) in query_args.iter().enumerate() {
-            if i > 0 {
-                query.push('&');
-            }
-            query.push_str(n);
+        if !status.to_string().is_empty() {
+            query_args.push(("status".to_string(), status.to_string()));
         }
+        let query_ = serde_urlencoded::to_string(&query_args).unwrap();
         let url = format!(
             "/repos/{}/{}/check-suites/{}/check-runs?{}",
             crate::progenitor_support::encode_path(&owner.to_string()),
             crate::progenitor_support::encode_path(&repo.to_string()),
             crate::progenitor_support::encode_path(&check_suite_id.to_string()),
-            query
+            query_
         );
 
         self.client.get(&url, None).await
@@ -437,34 +417,32 @@ impl Checks {
         page: i64,
         app_id: i64,
     ) -> Result<crate::types::ChecksListRefResponse> {
-        let mut query = String::new();
-        let mut query_args: Vec<String> = Default::default();
+        let mut query_args: Vec<(String, String)> = Default::default();
         if app_id > 0 {
-            query_args.push(format!("app_id={}", app_id));
+            query_args.push(("app_id".to_string(), app_id.to_string()));
         }
         if !check_name.is_empty() {
-            query_args.push(format!("check_name={}", check_name));
+            query_args.push(("check_name".to_string(), check_name.to_string()));
+        }
+        if !filter.to_string().is_empty() {
+            query_args.push(("filter".to_string(), filter.to_string()));
         }
-        query_args.push(format!("filter={}", filter));
         if page > 0 {
-            query_args.push(format!("page={}", page));
+            query_args.push(("page".to_string(), page.to_string()));
         }
         if per_page > 0 {
-            query_args.push(format!("per_page={}", per_page));
+            query_args.push(("per_page".to_string(), per_page.to_string()));
         }
-        query_args.push(format!("status={}", status));
-        for (i, n) in query_args.iter().enumerate() {
-            if i > 0 {
-                query.push('&');
-            }
-            query.push_str(n);
+        if !status.to_string().is_empty() {
+            query_args.push(("status".to_string(), status.to_string()));
         }
+        let query_ = serde_urlencoded::to_string(&query_args).unwrap();
         let url = format!(
             "/repos/{}/{}/commits/{}/check-runs?{}",
             crate::progenitor_support::encode_path(&owner.to_string()),
             crate::progenitor_support::encode_path(&repo.to_string()),
             crate::progenitor_support::encode_path(&ref_.to_string()),
-            query
+            query_
         );
 
         self.client.get(&url, None).await
@@ -501,32 +479,26 @@ impl Checks {
         per_page: i64,
         page: i64,
     ) -> Result<crate::types::ChecksListSuitesRefResponse> {
-        let mut query = String::new();
-        let mut query_args: Vec<String> = Default::default();
+        let mut query_args: Vec<(String, String)> = Default::default();
         if app_id > 0 {
-            query_args.push(format!("app_id={}", app_id));
+            query_args.push(("app_id".to_string(), app_id.to_string()));
         }
         if !check_name.is_empty() {
-            query_args.push(format!("check_name={}", check_name));
+            query_args.push(("check_name".to_string(), check_name.to_string()));
         }
         if page > 0 {
-            query_args.push(format!("page={}", page));
+            query_args.push(("page".to_string(), page.to_string()));
         }
         if per_page > 0 {
-            query_args.push(format!("per_page={}", per_page));
-        }
-        for (i, n) in query_args.iter().enumerate() {
-            if i > 0 {
-                query.push('&');
-            }
-            query.push_str(n);
+            query_args.push(("per_page".to_string(), per_page.to_string()));
         }
+        let query_ = serde_urlencoded::to_string(&query_args).unwrap();
         let url = format!(
             "/repos/{}/{}/commits/{}/check-suites?{}",
             crate::progenitor_support::encode_path(&owner.to_string()),
             crate::progenitor_support::encode_path(&repo.to_string()),
             crate::progenitor_support::encode_path(&ref_.to_string()),
-            query
+            query_
         );
 
         self.client.get(&url, None).await
